/*
 * sys/socket.h
 * Modified by K.Shirakata
 * Modified by Kazuya 'Sharl' Masuda
 */
#ifndef	__sys_socket_h__
#define	__sys_socket_h__

#include <sys/types.h>

#define	AF_UNSPEC	0
#define	AF_UNIX		1
#define	AF_LOCAL	1	/* ESP/X depends */
#ifdef	XIP_EXTENSION
#define	AF_INET		2
#else
#define	AF_INET		0	/* ESP/X depends */
#endif

#define	SOCK_STREAM	0
#define	SOCK_DGRAM	1
#define	SOCK_RAW	2

#define	SOCK_BINARY	0		/* socket in raw (binary) mode */
#define	SOCK_ASCII	1		/* socket in cooked (newline mapping) mode */
#define	SOCK_QUERY	2		/* Return setting without change */

#define	LOCDFLOW	5		/* dgram socket flow-control point, packets */
#define	LOCSFLOW	2048	/* stream socket flow control point, bytes */
#define	SOBUF		256		/* Size of buffer for usputc()/usprintf() */
#define	SOCKBASE	128		/* Start of socket indexes */

typedef struct usock {
  int refcnt;
  char noblock;
  char type;
  int rdysock;
  void *p;
  char *name;
  int namelen;
  char *peername;
  int peernamelen;
  char errcodes[4];		/* Protocol-specific error codes */
  void *obuf;			/* Output buffer */
  void *ibuf;			/* Input buffer */
  char eol[3];			/* Text mode end-of-line sequence, if any */
  int flag;				/* Mode flags, defined in socket.h */
  int flush;			/* Character to trigger flush, if any */
  int event;
} usock;

struct sockaddr {
  short sa_family;
  char sa_data[14];
};

int accept(int, struct sockaddr *, int *);
int bind(int, struct sockaddr *, int);
int connect(int, struct sockaddr *, int);
int getpeername(int, struct sockaddr *, int *);
int getsockname(int, struct sockaddr *, int *);
int listen(int, int);
int socket(int, int, int);
int recv(int, void *, size_t, int);
int recvfrom(int, void *, size_t, int, struct sockaddr *, int *);
int send(int, const void *, size_t, int);
int sendto(int, const void *, size_t, int, const struct sockaddr *, int);

int socklen(int, int);
int setflush(int, int);
int shutdown(int, int);
int sockkick(int);
int usesock(int);
int recvchar(int);
int recvline(int, char *, unsigned int);
int rrecvchar(int);
int seteol(int, char *);
int sockmode(int, int);
void usflush(int s);
char *psocket(void *);
char *sockerr(int);
char *sockstate(int);

#endif
